from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands

class LTVUptime(Sensor):

	def __init__(self, interval = "50", format = "%dd %hh %mm", u = "1", t = "0", n = "0", os="0", k = "0", r = "0", p="0" ):
		global _; _ = i18n.Translator("ltv-uptime-sensor")
		if n == "1" or os == "1" or "k" == "1" or r == "1" or p == "1" :
			c = "1"
		else:
			c = "0"
		self.__enable = [u,t,c,[n,os,k,r,p]]
		self.__uptime = "0d 0h 0m"
		Sensor.__init__(self)
		self._set_config_type("thermal", TYPE_STRING, t)
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("format", TYPE_STRING, format)
		self._set_config_type("host", TYPE_STRING, commands.getoutput("uname -n"))
		self._set_config_type("os", TYPE_STRING, commands.getoutput("uname -s"))
		self._set_config_type("kernel", TYPE_STRING, commands.getoutput("uname -s"))
		self._set_config_type("version", TYPE_STRING, commands.getoutput("uname -r"))
		self._set_config_type("processor", TYPE_STRING, commands.getoutput("uname -p"))
		if u != "0" or t != "0":
			self._add_timer(0, self.uptime_loop)
		self._add_timer(0, self.run_first)
		self._watch_config(self.watch_config)

	def get_configurator(self):
		configurator = self._new_configurator()
		u = self.__enable[ 0 ]
		t = self.__enable[ 1 ]
		c = self.__enable[ 2 ]
		if u == "0" and t == "0" and c == "0":
			return configuratior
		configurator.set_name(_("System info"))
		if u != "0" or t != "0" :
			configurator.add_title(_("Display"))
			configurator.add_spin(_("Update every (ds):"), "update", _("Updates the display every x deciseconds"), 1, 600)
		if u != "0" :
			configurator.add_title(_("Uptime"))
			configurator.add_entry(_("Format:"), "format", _("%d\tDays\n%h\tHours\n%m\tMinutes\n%M\tMinutes (2 digits)\n%s\tSeconds\n%S\tSeconds (2 digits)"))
		if t != "0" :
			configurator.add_title(_("Termal info"))
			configurator.add_option(_("Units:"), "thermal", _("Units to display temperature in."),[[_("Celcius"),"c"],[_("Fahrenheit"),"f"],[_("Kelvin"),"k"]])
		if c == "1" :
			configurator.add_title(_("General"))
			if self.__enable[ 3 ][ 0 ] != "0" :
				configurator.add_entry(_("Host name:"), "host", _("Your name on a network."))
			if self.__enable[ 3 ][ 1 ] != "0" :
				configurator.add_entry(_("OS name:"), "os", _("Your operating system."))
			if self.__enable[ 3 ][ 2 ] != "0" :
				configurator.add_entry(_("Kernel name:"), "kernel", _("Your system's kernel."))
			if self.__enable[ 3 ][ 3 ] != "0" :
				configurator.add_entry(_("Kernel version:"), "version", _("Your system's kernel's version."))
			if self.__enable[ 3 ][ 4 ] != "0" :
				configurator.add_entry(_("Processor name:"), "processor", _("Your system's processor."))
		return configurator

	def draw_uptime(self, data):
		if self.__enable[ 0 ] != "0" :
			uptime = time.time() - float(commands.getoutput("sysctl kern.boottime").split()[4][:-1])
			d = int( uptime / 60 / 60 / 24 )
			uptime = uptime - d * 60 * 60 * 24
			h = int( uptime / 60 / 60 )
			uptime = uptime - h * 60 * 60
			m = int( uptime / 60 )
			uptime = uptime - m * 60
			s = int( uptime )
			M = "%(m)02d" % vars()
			S = "%(s)02d" % vars()
			data.set("uptime",self._get_config("format").replace("%d",str(d)).replace("%h",str(h)).replace("%m",str(m)).replace("%s",str(s)).replace("%M",M).replace("%S",S))

	def draw_temperature(self, data):
		if self.__enable[ 1 ] != "0" :
			t = ""
			if self.__enable[ 1 ] != "0" :
				t = commands.getoutput("acpi -B -t -" + self._get_config("thermal"))
				if t[5:][:7] == "Thermal" :
					t = str( int( float( t.split()[ 3 ] ) ) )
					if self._get_config("thermal") == "f" :
						t = t + "&#176;F"
					elif self._get_config("thermal") == "k" :
						t = t + " K"
					else:
						t = t + "&#176;C"
				else:
					t = ""
			data.set("temperature", t)

	def uptime_loop(self):
		data = self._new_output()
		self.draw_uptime(data)
		self.draw_temperature(data)
		self._send_output(data)
		self._add_timer(self._get_config("update")*100, self.uptime_loop)

	def run_first(self):
		data = self._new_output()
		data.set("host",self._get_config("host"))
		data.set("os",self._get_config("os"))
		data.set("kernel",self._get_config("kernel"))
		data.set("version",self._get_config("version"))
		data.set("processor",self._get_config("processor"))
		self._send_output(data)

	def watch_config(self,key,value):
		data = self._new_output()
		if key == "format" :
			self.draw_uptime(data)
		elif key == "thermal" :
			self.draw_temperature(data)
		else:
			data.set(key, value)
		self._send_output(data)

def new_sensor(args): return apply(LTVUptime, args)
